// David Denny, 1001915603

// 12-bit GCLA
// Note that this is altered from the 8-bit Variation used for Lab 3
module TwelveBitGCLAadder (
	input [11:0] Aout, Bout,
	output [11:0] R
	);
   
	// Establish Internal Wires
	wire Cfour, Ceight, Ctwelve;
	wire GthreeZero, GsevenFour, GelevenEight;
	wire PthreeZero, PsevenFour, PelevenEight;

	// Stage 0, 4-Bit CLA for R[3:0]
	FourBitCLAadder CLA_inst_0 (
		.A(Aout[3:0]),
		.B(Bout[3:0]),
		.Cin(1'b0),
        .S(R[3:0]),
		.Gout(GthreeZero),
		.Pout(PthreeZero),
		.Cout() // Cout not used, C[4] calculated below
	);
	// Assign Carry for Stage 1
	assign Cfour = GthreeZero | (PthreeZero & 1'b0);

	// Stage 1, 4-Bit CLA for R[7:4]
	FourBitCLAadder CLA_inst_1 (
		.A(Aout[7:4]),
		.B(Bout[7:4]),
		.Cin(Cfour),
      .S(R[7:4]),
		.Gout(GsevenFour),
		.Pout(PsevenFour),
		.Cout() // Cout not used, C[8] calculated below
	);
	// Assign Carry for Stage 2
	assign Ceight = GsevenFour | (PsevenFour & Cfour);

	// Stage 2, 4-Bit CLA for R[11:0]
	// Note that R[11] Serves as the Cout for the
	// Exponent Incrementer and Normalizer.
	FourBitCLAadder CLA_inst_2 (
		.A(Aout[11:8]),
		.B(Bout[11:8]),
		.Cin(Ceight),
      .S(R[11:8]),
		.Gout(GelevenEight),
		.Pout(PelevenEight),
		.Cout() // Cout not used, R[11] is Cout for the GCLA
	);

endmodule

// 4-bit Carry Lookahead Adder (CLA)
module FourBitCLAadder (
	input [3:0] A, B,
	input Cin,
	output [3:0] S,
	output Gout, Pout, Cout
	);
	
	// Establish Internal Wires
	wire [3:0] G, P, C;

	// Generate and propagate signals for each bit
	assign G[0] = A[0] & B[0];
	assign P[0] = A[0] ^ B[0];
	assign G[1] = A[1] & B[1];
	assign P[1] = A[1] ^ B[1];
	assign G[2] = A[2] & B[2];
	assign P[2] = A[2] ^ B[2];
	assign G[3] = A[3] & B[3];
	assign P[3] = A[3] ^ B[3];

	// Carry signals
	assign C[0] = Cin;
	assign C[1] = G[0] | (P[0] & C[0]);
	assign C[2] = G[1] | (P[1] & C[1]);
	assign C[3] = G[2] | (P[2] & C[2]);

	// Sum outputs
	assign S[0] = P[0] ^ C[0];
	assign S[1] = P[1] ^ C[1];
	assign S[2] = P[2] ^ C[2];
	assign S[3] = P[3] ^ C[3];

	// Group generate and propagate
	assign Gout = G[3] | (P[3] & G[2]) | (P[3] & P[2] & G[1]) | (P[3] & P[2] & P[1] & G[0]);
	assign Pout = P[3] & P[2] & P[1] & P[0];
	assign Cout = C[3];
	
endmodule

